
/*
CHANGELOG:
// CHANGED 2004-07-28 Lutz Rogowski
// CHANGED 2004-07-21 Lutz Rogowski

CHANGED 2004-07-19 Lutz Rogowski
*/

#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <signal.h>

//#include <qtopia/config.h>
//#include <kconfig.h>

#include "../dissipate2/sipprotocol.h"
#include "../dissipate2/sipcall.h"
#include "../dissipate2/sdp.h"
#include "dspaudioout.h"
#include "dspaudioin.h"
#include "dspoutrtp.h"
#include "dspoutoss.h"
#include "callaudio.h"

CallAudio::CallAudio( void )
{
	output = 0;
	input = 0;
	curcall = 0;
	curmember = 0;
	useoss = true;
	setOSSFilename( "/dev/dsp" );
	setOSSFilename2( "/dev/dsp" );
	setOSSMode("readwrite");
	setOSSFilenameFS(7);
	setOSSFilename2FS(7);
	setVideoSW( QString::null );
	local.setIpAddress( "0.0.0.0" );
        
	payload = 160;//KCallSipSettings::sizeOfPayload();
	
	codec = codecILBC;
	rtpCodec = codecUnknown;
	videoCodec = codecUnknown;
	videoRtpCodec = codecUnknown;
	bodyMask = "";
	useStun = false;
	symMedia = false;
	pidVideo = 0;
	audio_fd = -1;
}

CallAudio::~CallAudio( void )
{
 
	if( output ) {
		if( output->running() ) {
			output->setCancel();
			output->wait();
		}
		delete output;
	}
	if( input ) {
		if( input->running() ) {
			input->setCancel();
			input->wait();
		}
		delete input;
	}
}

void CallAudio::useOSS( void )
{
	if( useoss == true ) return;
	useoss = true;
	renegotiateCall();
}

void CallAudio::setOSSFilename( const QString &devname )
{
    //qDebug("CallAudio::setOSSFilename %s ", devname.latin1());
	if( devname == QString::null ) {
		ossfilename = "/dev/dsp";
	} else {
		ossfilename = devname;
	}
}

void CallAudio::setOSSFilename2( const QString &devname )
{
	if( devname == QString::null ) {
		ossfilename2 = "/dev/dsp";
	} else {
		ossfilename2 = devname;
	}
}

void CallAudio::setVideoSW( const QString &sw )
{
	if( sw == QString::null ) {
		videoSW = "vic";
	} else {
		videoSW = sw;
	}
}

void CallAudio::audioIn( void )
{
	QString hostname = remote.getIpAddress();
	unsigned int portnum = remote.getPort();
	if( hostname == QString::null || portnum == 0 ) {
		printf( "CallAudio: SendToRemote called but we don't have a valid session description yet\n" );
		return;
	}
	if( remote.isOnHold() ) {
		printf( "CallAudio: Remote is currently putting us on hold, waiting patiently\n" );
		return;
	}
	printf( "CallAudio: Sending to remote site %s:%d\n",
		hostname.latin1(), portnum );
	if( input ) {
		stopListeningAudio();
	}
	DspOut *out;
	DspOut *in;
	if( useoss ) {
		DspOutRtp *outrtp;
		if (symMedia) {
			outrtp = new DspOutRtp( getRtpCodec(), getRtpCodecNum(), hostname, &socket );
		} else {
			outrtp = new DspOutRtp( getRtpCodec(), getRtpCodecNum(), hostname );
		}
		outrtp->setPortNum( portnum );
		outrtp->openDevice( DspOut::WriteOnly );
		//TODO why read settings again here?
		//payload = KCallSipSettings::sizeOfPayload();
		outrtp->setPayload( payload );
		//ossfilename2FS = KCallSipSettings::fragSizeRead();
		DspOutOss *inaudio = new DspOutOss( ossfilename2, ossfilename2FS  );
		if( /*KCallSipSettings::mode()*/ ossMode == "readwrite" ) {
			printf( "CallAudio: OSS device already open (readwrite)\n" );
			if( !inaudio->openDevice( audio_fd ) ) {
				printf( "** audioIn: openDevice Failed.\n" );
			}
		} else {
			printf( "CallAudio: Opening OSS device %s for Input \n", ossfilename2.latin1() );
			if( !inaudio->openDevice( DspOut::ReadOnly ) ) {
				printf( "** audioIn: openDevice Failed.\n" );
			}
		}
		inaudio->readBuffer(2);
		out = outrtp;
		in = inaudio;
		printf( "CallAudio: Creating OSS->RTP Diverter\n" );
		input = new DspAudioIn( in, out );
        qDebug("rettich ");
	}
	if( curcall->getCallType() == SipCall::videoCall ) {
            qDebug("QSipTel does not support video calls");
        /**** FIXME: we don't support video calls
		printf( "CallAudio: Opening SW for video input and output\n" );
		pidVideo = fork();
		if( !pidVideo ) {
			QString videoSW = KCallSipSettings::videoSW()
			//	"videoSW", "/usr/local/bin/vic" );
			QString videoCodec = getVideoRtpCodecName();
			QString SW = videoSW;
			if( SW.contains( "/" ) ) {
				SW = SW.mid( SW.findRev( '/' ) + 1 );
			}
			QString videoSWParam = hostname + "/" +
				QString::number( remote.getVideoPort() ) + "/";
			videoSWParam += videoCodec + "/16/" +
				QString::number( local.getVideoPort() );
			printf( "CallAudio: execlp( %s, %s, %s, 0)\n",
				videoSW.latin1(), SW.latin1(), videoSWParam.latin1() );
			execlp( videoSW.latin1(), SW.latin1(), videoSWParam.latin1(), 0 );
			printf("error executing " + videoSW + "\n");
			exit(1);
		}
                ****/
	}
}

void CallAudio::stopListeningAudio( void )
{
	if( input ) {
		if( input->running() ) {
			input->setCancel();
			input->wait();
		}
		delete input;
	}
	input = 0;
	remote = SdpMessage::null;
}

SdpMessage CallAudio::audioOut( void )
{
	printf( "CallAudio: listening for incomming RTP\n" );
	if( output ) {
		stopSendingAudio();
	}
	DspOut *out;
	DspOut *in;
	local.setIpAddress( Sip::getLocalAddress() );
	local.setName( "The Funky Flow" );
	if( useoss ) {
		DspOutRtp *inrtp;
		if (symMedia) {
			inrtp = new DspOutRtp( getRtpCodec(),
				getRtpCodecNum(), QString::null, &socket );
		} else {
			inrtp = new DspOutRtp( getRtpCodec(), getRtpCodecNum() );
		}
        //TODO 
		//payload = KCallSipSettings::sizeOfPayload();
		inrtp->setPayload( payload );
		if( useStun ) {
			inrtp->setStunSrv( stunSrv );
		}
		inrtp->openDevice( DspOut::ReadOnly );
		local.setPort( inrtp->getPortNum() );
		local.setVideoPort( inrtp->getVideoPortNum() );
        //ossfilenameFS = KCallSipSettings::fragSizeWrite();
		DspOutOss *outoss = new DspOutOss( ossfilename ,ossfilenameFS );
		if( /*KCallSipSettings::mode()*/ ossMode == "readwrite" ) {
			printf( "CallAudio: Opening OSS device %s for Input and Output\n", ossfilename.latin1() );
			if( !outoss->openDevice( DspOut::ReadWrite ) ) {
				printf( "** audioOut: openDevice Failed.\n" );
			} else {
				audio_fd = outoss->audio_fd;
			}
		} else {
			printf( "CallAudio: Opening OSS device %s for Output\n", ossfilename.latin1() );
			if( !outoss->openDevice( DspOut::WriteOnly )){
				printf( "** audioOut: openDevice Failed.\n" );
			}
		}
		in = inrtp;
		out = outoss;
		printf( "CallAudio: Creating RTP->OSS Diverter\n" );
		output = new DspAudioOut( in, out );
	}
	return local;
}

void CallAudio::stopSendingAudio( void )
{
	if( output ) {
		if( output->running() ) {
			output->setCancel();
			output->wait();
		}
		delete output;
	}
	output = 0;
	local.setIpAddress( "0.0.0.0" );
	local.setName( "Idle" );
	local.setPort( 0 );
	local.setVideoPort( 0 );
}

void CallAudio::readAudioSettings( void )
{
	useOSS();
	//setOSSFilename( KCallSipSettings::ossfilename() );
	//setOSSFilename2( KCallSipSettings::ossfilename2() );
	//if( KCallSipSettings::media() == "Yes" ) {
	//	setSymMediaMode( true );
	//}
	//ossfilenameFS = KCallSipSettings::fragSizeWrite();
	//ossfilename2FS = KCallSipSettings::fragSizeRead();
}

void CallAudio::readVideoSettings( void )
{
/**** FIXME: no video settings
	setVideoSW( KCallSipSettings::videoSW", "/usr/local/bin/vic"  ) );
        ***/
}

void CallAudio::setCurrentCall( SipCall *newcall )
{
	curcall = newcall;
	statusUpdated();
}

void CallAudio::attachToCallMember( SipCallMember *newmember )
{
	if( curmember ) {
		detachAndHold();
	}
	curmember = newmember;
	connect( curmember, SIGNAL( statusUpdated(SipCallMember *) ),
		this, SLOT( memberStatusUpdated(SipCallMember *) ) );
	statusUpdated();
}

void CallAudio::detachAndHold( void )
{
	if( local.isOnHold() ) {
		printf( "CallAudio: Call already on hold\n" );
	} else {
		toggleOnHold();
	}
}

void CallAudio::toggleOnHold( void )
{
	if( local.isOnHold() ) {
		printf( "CallAudio: Resuming call\n" );
		curmember->requestInvite(
			audioOut().message( getRtpCodec(), getVideoRtpCodec(), getBodyMask() ),
			MimeContentType( "application/sdp" ) );
	} else {
		printf( "CallAudio: Putting call on hold\n" );
		local.setIpAddress( "0.0.0.0" );
		local.setName( "Whoa there dewd" );
		local.setPort( 0 );
		local.setVideoPort( 0 );
		curmember->requestInvite(
			local.message( getRtpCodec(), getVideoRtpCodec(), getBodyMask() ),
			MimeContentType( "application/sdp" ) );
		detachFromCall();
	}
}

//TODO this should be moved to the sip-stack
bool CallAudio::checkCodec( SipCallMember *member )
{
	bool status = true;
	QString mstr = member->getSessionDescription();
	rtpCodec = codecUnknown;
	videoRtpCodec = codecUnknown;
	if( mstr.contains( "m=audio" ) ) {
		QString ilbc = "";
		QString m = mstr.mid( mstr.find( "m=audio" ) );
		m = m.left( m.find( "\n" ) );
		m = m.mid( m.find( "RTP/AVP" ) + 7 );
		m += ' ';
		if( mstr.lower().contains( "ilbc/8000" ) ) {
			ilbc = mstr.mid( mstr.lower().find( "ilbc/8000" ) - 7, 6 );
			if( ilbc.contains( ":" ) ) {
				ilbc = ilbc.mid( ilbc.find( ":" ) + 1 );
			}
			ilbc = ilbc.simplifyWhiteSpace();
		}
		int posULAW = m.find( " 0 " );
		int posGSM = m.find( " 3 " );
		int posILBC = m.lower().find( " " + ilbc + " " );
		if( posULAW < 0 ) posULAW = 101;
		if( posGSM < 0 ) posGSM = 101;
		if( posILBC < 0 ) posILBC = 101;
		if( posULAW < posGSM && posULAW < posILBC ) {
			rtpCodec = codecULAW;
			rtpCodecNum = 0;
		} else if( posGSM < posULAW && posGSM < posILBC ) {
			rtpCodec = codecGSM;
			rtpCodecNum = 3;
		} else if( posILBC < posULAW && posILBC < posGSM ) {
			rtpCodec = codecILBC;
			rtpCodecNum = ilbc.toInt();
		}
	}
	if( mstr.contains( "m=video" ) ) {
		QString ilbc = "";
		QString m = mstr.mid( mstr.find( "m=video" ) );
		m = m.left( m.find( "\n" ) );
		m = m.mid( m.find( "RTP/AVP" ) + 7 );
		m += ' ';
		int posH261 = m.find( " 31 " );
		int posH263 = m.find( " 34 " );
		if( posH261 < 0 ) posH261 = 101;
		if( posH263 < 0 ) posH263 = 101;
#ifdef MESSENGER
		if( videoCodec == codecH261 ) {
			videoRtpCodec = codecH261;
			videoRtpCodecNum = 31;
		} else {
			videoRtpCodec = codecH263;
			videoRtpCodecNum = 34;
		}
#else
		if( posH261 < posH263 ) {
			videoRtpCodec = codecH261;
			videoRtpCodecNum = 31;
		} else if( posH263 < posH261 ) {
			videoRtpCodec = codecH263;
			videoRtpCodecNum = 34;
		}
#endif
	}

	if( rtpCodec == codecILBC ) {
		printf( "CallAudio: Using iLBC for output\n" );
	} else if( rtpCodec == codecGSM ) {
		printf( "CallAudio: Using GSM for output\n" );
	} else if( rtpCodec == codecULAW ) {
		printf( "CallAudio: Using G711u for output\n" );
	} else if( rtpCodec == codecUnknown ) {
		status = false;
	}
	if( rtpCodec != codecUnknown ) {
		if( videoRtpCodec == codecH261 ) {
			printf( "CallAudio: Using H261 for video output\n" );
		} else if( videoRtpCodec == codecH263 ) {
			printf( "CallAudio: Using H263 for video output\n" );
		} else {
			if( curcall ) {
				curcall->setCallType( SipCall::StandardCall ); 
			}
		}
	}
	
	return status;
}

void CallAudio::memberStatusUpdated(SipCallMember *member)
{
	SdpMessage sdpm;
	SdpMessage rsdp;
	curmember = member;
	sdpm.parseInput( curmember->getSessionDescription() );
	if( checkCodec( curmember ) ) {
		if( curmember->getState() == SipCallMember::state_RequestingReInvite ) {
			if( sdpm.isOnHold() ) {
				rsdp.setName( "Accepting on hold" );
				rsdp.setIpAddress( "0.0.0.0" );
				rsdp.setPort( 0 );
				rsdp.setVideoPort( 0 );
				curmember->acceptInvite(
					rsdp.message( getRtpCodec(),
						getVideoRtpCodec(), getBodyMask() ),
					MimeContentType( "application/sdp" ) );
				stopSendingAudio();
				stopListeningAudio();
			} else {
				curmember->acceptInvite(
					audioOut().message( getRtpCodec(),
						getVideoRtpCodec(), getBodyMask() ),
					MimeContentType( "application/sdp" ) );
			}
		}
		if( sdpm != remote ) {
			remote = sdpm;
			if( !sdpm.isOnHold() ) {
				output->setCodec( getRtpCodec(), getRtpCodecNum() );
				audioIn();
				output->start();
				input->start();
			}
			statusUpdated();
		}
	}
}

void CallAudio::detachFromCall( void )
{
    //qDebug("CallAudio::detachFromCall ");
	if( curmember ) {
		disconnect( curmember, 0, this, 0 );
	}
	curmember = 0;
	stopSendingAudio();
	stopListeningAudio();
	setCurrentCall( 0 );
	if( pidVideo ) {
		kill( pidVideo, SIGKILL );
		pidVideo = 0;
	}
}

bool CallAudio::isRemoteHold( void )
{
	return remote.isOnHold();
}

void CallAudio::renegotiateCall( void )
{
	if( !curcall ) return;
	stopSendingAudio();
	stopListeningAudio();
	curmember->requestInvite(
		audioOut().message( getRtpCodec(),
			getVideoRtpCodec(), getBodyMask() ),
		MimeContentType( "application/sdp" ) );
}

bool CallAudio::isAudioOn( void )
{
	return (output || input );
}

codecType CallAudio::getRtpCodec( void )
{
	if( rtpCodec != codecUnknown ) {
		return rtpCodec;
	} else {
		return codec;
	}
}

int CallAudio::getRtpCodecNum( void )
{
	int c;
	if( rtpCodec != codecUnknown ) {
		c = rtpCodecNum;
	} else {
		switch( codec ) {
			case codecGSM:
				c = 3;
				break;
			case codecILBC:
				c = 97;
				break;
			case codecULAW:
			default:
				c = 0;
				break;
		}
	}
	return c;
}

codecType CallAudio::getVideoRtpCodec( void )
{
	if( videoRtpCodec != codecUnknown ) {
		return videoRtpCodec;
	} else {
		if( curcall ) {
			if( curcall->getCallType() == SipCall::videoCall ) {
				return videoCodec;
			} else {
				return codecUnknown;
			}
		} else {
			return codecUnknown;
		}
	}
}

int CallAudio::getVideoRtpCodecNum( void )
{
	int c;
	if( curcall ) {
		if( curcall->getCallType() != SipCall::videoCall ) {
			return -1;
		}
	}
	if( videoRtpCodec != codecUnknown ) {
		c = videoRtpCodecNum;
	} else {
		switch( videoCodec ) {
			case codecH261:
				c = 31;
				break;
			case codecH263:
				c = 34;
				break;
			default:
				c = -1;
				break;
		}
	}
	return c;
}

QString CallAudio::getVideoRtpCodecName( void )
{
	QString c;
	if( curcall->getCallType() != SipCall::videoCall ) {
		return "";
	}
	if( videoRtpCodec != codecUnknown ) {
		switch( videoRtpCodec ) {
			case codecH263:
				c = "h263";
				break;
			default:
			case codecH261:
				c = "h261";
				break;
		}
	} else {
		switch( videoCodec ) {
			case codecH263:
				c = "h263";
				break;
			default:
			case codecH261:
				c = "h261";
				break;
		}
	}
	return c;
}

void CallAudio::setStunSrv( QString newStunSrv )
{
	useStun = true;
	stunSrv = newStunSrv;
}

void CallAudio::startDTMF(char code)
{
	if (output) {
		output->startTone(code);
	}

	if (input) {
		input->startTone(code);
	}
}

void CallAudio::stopDTMF(void)
{
	if (output) {
		output->stopTone();
	}

	if (input) {
		input->stopTone();
	}
}



#include "callaudio.moc"
