// CHANGED 2004-07-28 Lutz Rogowski
#include <stdio.h>
#include <unistd.h>

#include "audiobuffer.h"
#include "dspout.h"
#include "dspoutrtp.h"
#include "dspaudioin.h"
#include <qapplication.h>

DspAudioIn::DspAudioIn( DspOut *in, DspOut *out )
	: DspAudio( in, out )
{
	udp_failures = 0;
}

DspAudioIn::~DspAudioIn( void )
{
}

void DspAudioIn::timerTick( void )
{

	if( broken ) {
		printf("(B)");
		fflush(stdout);
		return;
	}

	int bytes = input->readableBytes();

	if( bytes > 0 ) {

		// Read in a buffer
        int br = input->readBuffer( bytes );
        if ( br == 0 )
            return;
		// Make sure there's room in the output
        if ( br != bytes ) {
            qDebug("Bytes req %d, got %d ", bytes, br);
            input->getBuffer().resize( br );
        }
            
		output->getBuffer().resize( input->getBuffer().getSize() );
		if (!generateDTMF((short*)output->getBuffer().getData(),
			input->getBuffer().getSize() / sizeof(short))) {

			// Copy the data
          
			memcpy( output->getBuffer().getData(),
				input->getBuffer().getData(),
				input->getBuffer().getSize() );
        }

		// If the write fails, mark the pipe as broken and stop sending!
		if( !output->writeBuffer() ) {
			udp_failures++;
			if(udp_failures > 5){
				broken = true;
				printf( "DspAudioIn: Broken pipe\n" );
			} else {
				printf( "DspAudioIn: UDP-failure(%d)\n", udp_failures );
			}
		} else {
			udp_failures = 0;
		}
	}
}

#ifndef HAVE_THREAD
void DspAudioIn::start( void )
{
	timer = new QTimer();
	connect( timer, SIGNAL( timeout() ), this, SLOT( timeout() ) ); 
	timer->start( 1 );
}

void DspAudioIn::timeout()
{
	timerTick();
}

#endif

#include "dspaudioin.moc"
