// CHANGED 2004-07-28 Lutz Rogowski
// CHANGED 2004-07-21 Lutz Rogowski
/*
 * $Id: dtmfgenerator.cpp 386159 2005-02-04 16:18:44Z mhauth $
 *
 * (c) 2003 iptel.org
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 */

#include <math.h>
#include <qapplication.h>
#include <qprogressbar.h>
#include "dtmfgenerator.h"


/*
 * Tone frequencies
 */
const DTMFGenerator::DTMFTone DTMFGenerator::tones[NUM_TONES] = {
	{'0', 941, 1336},
	{'1', 697, 1209},
	{'2', 697, 1336},
	{'3', 697, 1477},
	{'4', 770, 1209},
	{'5', 770, 1336},
	{'6', 770, 1477},
	{'7', 852, 1209},
	{'8', 852, 1336},
	{'9', 852, 1477},
	{'A', 697, 1633},
	{'B', 770, 1633},
	{'C', 852, 1633},
	{'D', 941, 1633},
	{'*', 941, 1209},
	{'#', 941, 1477}
};

short* DTMFGenerator::samples[NUM_TONES] ;  
bool DTMFGenerator::samplesCreated = false ;        

DTMFException::DTMFException(const char* _reason) throw()
{
	reason = _reason;
}


DTMFException::~DTMFException() throw()
{
}


const char* DTMFException::what() const throw()
{
	return reason;
}

bool DTMFGenerator::generateSamples()
{
    if ( !samplesCreated ) { 
        QProgressBar bar(NUM_TONES-1 ,0 );
        bar.setCaption ("KPhone/Pi: Generating Samples..."  ); 
        int w = 400;
        if ( QApplication::desktop()->width() < 320 )
            w = 220;
        int h = bar.sizeHint().height() ;
        int dw = QApplication::desktop()->width();
        int dh = QApplication::desktop()->height();
        bar.setGeometry( (dw-w)/2, (dh - h )/2 ,w,h );
        bar.show();
        samplesCreated = true;
        qDebug("DTMFGenerator::generateSamples");
        for(int i = 0; i < NUM_TONES; i++) {
            bar.setProgress(i);
            qApp->processEvents();
            if ( bar.isHidden() ) {
                int j;
                for(j = 0; j < i; j++) {
                    delete[] samples[j];
                }
                return false;
            }
            samples[i] = generateSample(i);
        }
        bar.setCaption ("KPhone/Pi: Starting..."  ); 
        qApp->processEvents();
    }
    return true;

}
void  DTMFGenerator::deleteSamples()
{
    if ( samplesCreated ) {
        qDebug("DTMFGenerator::deleteSamples() ");
        for(int i = 0; i < NUM_TONES; i++) {
            delete[] samples[i];
        }
    }
}


/*
 * Initialize the generator
 */
DTMFGenerator::DTMFGenerator()
{
	samplingRate = 8000;
   	amplitude = 8192;

	state.offset = 0;
	state.sample = 0;
    //generateSamples();
}
	

DTMFGenerator::~DTMFGenerator()
{

}


/*
 * Get n samples of the signal of code code
 */
void DTMFGenerator::getSamples(short* buffer, size_t n, unsigned char code) 
{
	size_t i;

	if (!buffer) {
        return;
	}

	switch(code) {
	case '0': state.sample = samples[0];  break;
	case '1': state.sample = samples[1];  break;
	case '2': state.sample = samples[2];  break;
	case '3': state.sample = samples[3];  break;
	case '4': state.sample = samples[4];  break;
	case '5': state.sample = samples[5];  break;
	case '6': state.sample = samples[6];  break;
	case '7': state.sample = samples[7];  break;
	case '8': state.sample = samples[8];  break;
	case '9': state.sample = samples[9];  break;
	case 'A':
	case 'a': state.sample = samples[10]; break;
	case 'B':
	case 'b': state.sample = samples[11]; break;
	case 'C':
	case 'c': state.sample = samples[12]; break;
	case 'D':
	case 'd': state.sample = samples[13]; break;
	case '*': state.sample = samples[14]; break;
	case '#': state.sample = samples[15]; break;
	default:
		return;
		break;
	}


    int mod = samplingRate/2;
    int rest = n;
    short* to = buffer;
    short* from ;
    while ( rest >= mod ) {
        from = &state.sample[0];
        for(i = 0; i < mod; i++) *to++ = *from++;
        rest -= mod;
    }
    if ( rest ) {
        from = &state.sample[0];
        for(i = 0; i < rest; i++) *to++ = *from++;
    }
    
	state.offset = rest;

#if 0
	for(i = 0; i < n; i++) {
		buffer[i] = state.sample[i % samplingRate];
	}

	state.offset = i % samplingRate;
#endif

}


/*
 * Get next n samples (continues where previous call to
 * genSample or genNextSamples stopped
 */
void DTMFGenerator::getNextSamples(short* buffer, size_t n) 
{
	size_t i;

	if (!buffer) {
		return;
	}

	if (state.sample == 0) {
		return;
	}

    
    int mod = samplingRate/2;
    int rest = n;
    short* to = buffer;
    short* from ;
    int first = mod - state.offset;
    if ( rest >= first ) {
        from = &state.sample[state.offset];
        for(i = 0; i < first; i++) *to++ = *from++;
        rest -= first;
        state.offset = 0;
    }
    while ( rest >= mod ) {
        from = &state.sample[0];
        for(i = 0; i < mod; i++) *to++ = *from++;
        rest -= mod;
    }
    if ( rest ) {
        from = &state.sample[state.offset];
        for(i = 0; i < rest; i++) *to++ = *from++;
    }
    state.offset = (state.offset+rest);



#if 0
	for(i = 0; i < n; i++) {
		buffer[i] = state.sample[(state.offset + i) % samplingRate];
	}
	
	state.offset = (state.offset + i) % samplingRate;

#endif 
}


/*
 * Generate a tone sample
 */
short* DTMFGenerator::generateSample(unsigned char code) 
{
	short* ptr;
	double arg1, arg2;


    ptr = new short[samplingRate/2];
    if (!ptr) {
        return 0;
    }

    arg1 = (double)2 * (double)M_PI * (double)tones[code].higher / (double)samplingRate; 
    arg2 = (double)2 * (double)M_PI * (double)tones[code].lower / (double)samplingRate;
    int max = samplingRate/2;
    for(unsigned int i = 0; i < max; i++) {
        ptr[i] = (short)((double)(amplitude >> 2) * sin(arg1 * i) + (double)(amplitude >> 2) * sin(arg2 * i));
    };
		
    return ptr;

}
