/*
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/
//This file was modified by Mike Hauth and Eva Brucherseifer


#include "callmanager.h"
#include "kcallsipsettings.h"

#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <qdatetime.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qtimer.h>
#include <qdir.h>
#include <qapplication.h>
#include <qmessagebox.h>
#include <sys/soundcard.h>
#include <sys/ioctl.h>
#include <math.h>
#include <qgrid.h>
#include <qsound.h>
#include <qbuttongroup.h>
#include <qdatetime.h>

#include "../dissipate2/sdp.h"
#include "../dissipate2/sipmessage.h"
#include "../dissipate2/mimecontenttype.h"
#include "../dissipate2/sipuser.h"
#include "../dissipate2/sipcall.h"
#include "../dissipate2/siptransaction.h"
#include "ksipauthentication.h"
#include "../audio/callaudio.h"
#include "../audio/kphoneconst.h"

#include <kaudioplayer.h>
#include <knotifyclient.h>

#include <kdebug.h>

extern QString globalIconPath;
extern int global_fd_led;
#ifndef DESKTOP_VERSION
////#include <qtopia/sound.h>
#include <kconfig.h>
////#include <qtopia/qpeapplication.h>
/*///#include <qtopia/qcopenvelope_qws.h>*/
#include <kstandarddirs.h>
#endif


CallManager::CallManager( KSipAuthentication *auth, CallAudio *callaudio, SipCall *initcall,
                          bool incoming, QWidget *parent, const char *name )
        : QObject( parent, name ), call( initcall ), sipauthentication(auth)
{

    kdDebug() << "CallManager::CallManager()" << endl;
    
    curstate = PreDial;
    member = 0;
//    useWav = false;
    isIncomingCall=incoming;

    audio = callaudio;
    connect( audio, SIGNAL( outputDead() ),
             this, SLOT( audioOutputDead() ) );
    connect( audio, SIGNAL( statusUpdated() ),
             this, SLOT( updateAudioStatus() ) );

//    ringCount = 0;
//    ringTimer = new QTimer();
//    connect( ringTimer, SIGNAL( timeout() ), this, SLOT( ringTimeout() ) );
    acceptCallTimer = new QTimer();
    connect( acceptCallTimer, SIGNAL( timeout() ),
             this, SLOT( acceptCallTimeout() ) );

    switchCall( initcall );
}

CallManager::~CallManager( void )
{
    kdDebug() << "CallManager::~CallManager()" << endl;
//    delete ringTimer;
    delete acceptCallTimer;
    delete call;
}

/*
void CallManager::keypadPressed( int id )
{
	char code = 0;
	switch(id) {
		case 0: code = '1'; break;
		case 1: code = '2'; break;
		case 2: code = '3'; break;
		case 3: code = '4'; break;
		case 4: code = '5'; break;
		case 5: code = '6'; break;
		case 6: code = '7'; break;
		case 7: code = '8'; break;
		case 8: code = '9'; break;
		case 9: code = '*'; break;
		case 10: code = '0'; break;
		case 11: code = '#'; break;
	}
    //qDebug("press %d ", id);
    if ( code != 0 ) {
        QString rt = remote->currentText()+ code ;
        remote-> lineEdit ()->setText(rt) ;
        audio->startDTMF(code);
    }
}
*/

/*
void CallManager::keypadReleased( int id )
{
	audio->stopDTMF();
}
*/

void CallManager::setRemote( QString newremote )
{
    kdDebug() << "CallManager::setRemote()" << endl;
    remote = newremote;
}


SipCall *CallManager::getCall()
{
    kdDebug() << "CallManager::getCall()" << endl;
    return call;
}

void CallManager::switchCall( SipCall *newcall )
{
    kdDebug() << "CallManager::switchCall()" << endl;
//    stopRing();
    kdDebug()<< "CallManager: Switching calls..." << endl;
    call = newcall;
    audio->setBodyMask( call->getSdpMessageMask() );
    if( member )
        disconnect( member, 0, this, 0 );
    member = call->getMemberList().toFirst();
    if( member )
    {
        if( member->getState() == SipCallMember::state_Disconnected )
        {
            forceDisconnect();
            return;
        }
        if( member->getState() == SipCallMember::state_Redirected )
        {
            handleRedirect();
            return;
        }
        connect( member, SIGNAL( statusUpdated( SipCallMember * ) ),
                 this, SLOT( callMemberStatusUpdated() ) );
        if( isIncomingCall )
        {
            if( call->getCallType() == SipCall::videoCall )
            {
                // FIXME: implement
            }
            else
            {
                // FIXME: implement
            }
            QString ss = member->getUri().uri();
            QDateTime t = QDateTime::currentDateTime();

            // Ringing tone
//            useWav = false;
//            isRingingTone = false;
            QString p = KCallSipSettings::ringingtone().upper();
            if ( p == "YES" || p == "WAV" )
            {
//                isRingingTone = true;
                //if ( p == "WAV" )
//                    useWav = true;
            }
//            ringCount = 0;
//            startRing ( );
        }
        remote = member->getUri().uri();
        subject = call->getSubject();
        if( member->getState() == SipCallMember::state_InviteRequested )
        {
            curstate = Calling;
        }
        else if( member->getState() == SipCallMember::state_RequestingInvite )
        {
            curstate = Called;
        }
        else
        {
            curstate = Connected;
        }
    }
    else
    {
        if( call->getCallType() == SipCall::videoCall )
        {
            // FIXME: implement
        }
        else
        {
            // FIXME: implement
        }
        subject = call->getSubject();
        remote = QString::null;
        curstate = PreDial;
        forceDisconnect();
    }
    updateAudioStatus();
}

void CallManager::dial( void )
{
    kdDebug() << "CallManager::dial()" << endl;
    // Multi-purpose buttons hack
    if( curstate == Called )
    {
        acceptCall();
        return;
    }
    if( remote.isEmpty())
    {
        QMessageBox::critical( 0, "Error: No Destination",
                               "You must specify someone to call." );
        return;
    }
    if( !audio->isAudioOn() )
    {
        QString devopen =  KCallSipSettings::ossfilename();
        audio_fd = ::open( devopen, O_WRONLY | O_NONBLOCK );
        if( audio_fd == -1 )
        {
            QMessageBox::information( 0, "Dial",
                                      "Close any program which might be using soundcard \nand then retry dial." );
            return;
        }
        else
        {
            ::close( audio_fd );
        }
    }
    QString strRemoteUri;
    QString s = remote;
    if( s.contains( '[' ) && s.contains( ']' ) )
        strRemoteUri = s.mid( s.find( '[' ) + 1, s.find( ']' ) - s.find( '[' ) - 1 );
    else
    {
        if( s.left( 4 ).lower() != "tel:" )
        {
            if( s.left( 4 ).lower() != "sip:" )
                s = "sip:" + s;
            if( !s.contains( '@' ) )
                s = s + "@" + call->getHostname();
        }
        strRemoteUri = s;
    }
    remote=s;
    qDebug("remote: %s", remote.latin1());
    // FIXME: add remote number to dialled
    SipUri remoteuri( strRemoteUri );
    audio->setRtpCodec( codecUnknown );
    audio->setVideoRtpCodec( codecUnknown );
    member = new SipCallMember( call, remoteuri );
    connect( member, SIGNAL( statusUpdated( SipCallMember * ) ),
             this, SLOT( callMemberStatusUpdated() ) );
    connect( member, SIGNAL( statusUpdated( SipCallMember * ) ),
             sipauthentication, SLOT( authRequest( SipCallMember * ) ) );
    audio->setCurrentCall( call );
    member->requestInvite(
        audio->audioOut().message( audio->getRtpCodec(), audio->getVideoRtpCodec(), audio->getBodyMask() ),
        MimeContentType( "application/sdp" ) );
    audio->attachToCallMember( member );
    curstate = Calling;

    // notify GUI
    QStringList members;
    SipCallMemberIterator it = call->getMemberList();
    while (++it)
        members.append( (*it)->getUri().uri() );
    // incomingCall parameters: callid, subject, member uris
    ////QCopEnvelope e("QPE/SipTel", "dialing(QString, QString, QStringList)");
    ////e << call->getCallId();
    ////e << call->getSubject();
    ////e << members;
}

void CallManager::callMemberStatusUpdated( void )
{
    kdDebug() << "CallManager::callMemberStatusUpdated()" << endl;
    SdpMessage sdpm;
    SdpMessage rsdp;
    // notify client
    // parameters: call id, status, status text
    ////QCopEnvelope e("QPE/SipTel", "callStatusUpdated(QString, int, QString)");
    ////e << call->getCallId();
    ////e << QString::number(member->getState());
    ////e << member->getLocalStatusDescription();
    qDebug("callStatusUpdated: %s %i %s", call->getCallId().latin1(), member->getState(), member->getLocalStatusDescription().latin1());
    if( member->getState() == SipCallMember::state_Disconnected )
    {
        //		if( member->getLocalStatusDescription().left( 12 ) == "Call Failed:" ) {
        //			curstat->setText( "Call Failed" );
        //                        ; // FIXME
        //            if ( isVisible () )
        //                QMessageBox::critical( this, "KPhone",
        //                                       member->getLocalStatusDescription() );
        //			prepareForDelete();
        //                        ; // FIXME
        //		} else {
        //			//curstat->setText( member->getLocalStatusDescription() );
        //                        ; // FIXME
        //		}
        //                */
        forceDisconnect();
    }
    else if( member->getState() == SipCallMember::state_Redirected )
    {
        //		curstat->setText( member->getLocalStatusDescription() );
        handleRedirect();
    }
    else if( member->getState() == SipCallMember::state_Refer )
    {
        //		curstat->setText( member->getLocalStatusDescription() );
        handleRefer();
    }
    else if( member->getState() == SipCallMember::state_Connected )
    {
        //		curstat->setText( member->getLocalStatusDescription() );
        curstate = Connected;
    }
    else
    {
        //		curstat->setText( member->getLocalStatusDescription() );
        ; // FIXME
    }
}

void CallManager::audioOutputDead( void )
{
    kdDebug() << "KCallAudio: Broken output pipe, disconnecting unpolitely\n" << endl;
    forceDisconnect();
}

void CallManager::hangupCall( void )
{
    kdDebug() << "CallManager::hangupCall()" << endl;
//    stopRing();
    if (!member)
    {
        prepareForDelete();
        return;
    }

    // Reject call if that's our current state
    if( curstate == Called )
    {
        member->declineInvite();
        prepareForDelete();
        return;
    }
    if( call->getCallStatus() != SipCall::callDead )
    {
        if( member->getState() == SipCallMember::state_Connected )
        {
            // qDebug("member->requestDisconnect() ");
            if( audio->getCurrentCall() == call )
            {
                audio->detachFromCall();
            }
            member->requestDisconnect();
        }
        else
        {
            //qDebug("member->cancelTransaction() ");
            if( audio->getCurrentCall() == call )
            {
                audio->detachFromCall();
            }
            member->cancelTransaction();
        }
        prepareForDelete();
        return;
    }
}

void CallManager::acceptCall( void )
{
    kdDebug() << "CallManager::acceptCall()" << endl;
//    stopRing();
    acceptCallTimer->start( acceptTime, true );
}

void CallManager::holdCall( void )
{
    kdDebug() << "CallManager::holdCall()" << endl;
    if( audio->getCurrentCall() == call )
    {
        audio->toggleOnHold();
    }
    else if( member )
    {
        audio->setCurrentCall( call );
        audio->attachToCallMember( member );
        audio->toggleOnHold();
    }
}

void CallManager::handleRedirect( void )
{
    kdDebug() << "CallManager::handleRedirect()" << endl;
    QValueList<SipUri>::Iterator it;
    SipUriList urilist = member->getRedirectList();
    while ( urilist.getListLength() > 0 )
    {
        SipUri redirto = urilist.getPriorContact();
        QMessageBox mb( "Redirect",
                        "Call redirected to: \n" + redirto.reqUri() + "\n\n" +
                        "Do you want to proceed ? ",
                        QMessageBox::Information,
                        QMessageBox::Yes | QMessageBox::Default,
                        QMessageBox::No,
                        QMessageBox::Cancel | QMessageBox::Escape );
        switch( mb.exec() )
        {
        case QMessageBox::No:
            continue;
        case QMessageBox::Cancel:
            forceDisconnect();
            return;
        }
        if( audio->getCurrentCall() == call )
        {
            audio->detachFromCall();
        }
        if( member )
            disconnect( member, 0, this, 0 );
        member = 0;
        
        redirectCall( redirto, subject );
        prepareForDelete();
        return;
    }
    forceDisconnect();
}

void CallManager::handleRefer( void )
{
    kdDebug() << "CallManager::handleRefer()" << endl;
    SipUri redirto = member->getRedirectList().getHead();
    if( audio->getCurrentCall() == call )
    {
        audio->detachFromCall();
    }
    QMessageBox mb( "Redirect",
                    "Call redirected to: \n" + redirto.reqUri() + "\n\n" +
                    "Do you want to proceed ? ",
                    QMessageBox::Information,
                    QMessageBox::Yes | QMessageBox::Default,
                    QMessageBox::No,
                    0 );
    if( mb.exec() == QMessageBox::Yes )
    {
        redirectCall( redirto, subject );
        prepareForDelete();
    }
    else
    {
        forceDisconnect();
    }
}

void CallManager::forceDisconnect( void )
{
    kdDebug() << "CallManager::forceDisconnect()" << endl;
    if( audio->getCurrentCall() == call )
    {
        audio->detachFromCall();
    }
    if( member )
        disconnect( member, 0, this, 0 );
    member = 0;

//    stopRing();
    prepareForDelete();
    
}

void CallManager::updateAudioStatus( void )
{
    kdDebug() << "CallManager::updateAudioStatus()" << endl;
    ////QCopEnvelope e("QPE/SipTel", "audioStatusUpdated(QString, int)");
    ////e << call->getCallId();
    ////if( audio->getCurrentCall() == call )
    ////{
    ////    if( audio->isRemoteHold() )
    ////        e << (int)AudioStatus::OnHold;  //[holding]
    ////    else
    ////        e << (int)AudioStatus::Active; // [active]
    ////}
    ////else
    ////{
    ////    e << (int)AudioStatus::Unattached; // Unattached
    ////}
}


void CallManager::pleaseTransfer( const QString &transferto )
{
    kdDebug() << "CallManager::pleaseTransfer()" << endl;
    // Ignore this button if there is no call setup yet.
    if( curstate != Connected )
        return;
    if( call->getCallStatus() != SipCall::callDead )
    {
        QString s = transferto;
        if( s.left( 4 ).lower() != "tel:" )
        {
            if( s.left( 4 ).lower() != "sip:" )
            {
                s = "sip:" + s;
            }
            if( !s.contains( '@' ) )
            {
                s = s + "@" +  member->getUri().getHostname();  //call->getHostname();
            }
        }
        member->requestTransfer( SipUri( s ) );
        return;
    }
}


/*void CallManager::stopRing()
{
    kdDebug() << "CallManager::stopRing()" << endl;
//    ringTimer->stop();

    system("artsshell terminate");    
    // FIXME: implement buttons for av500
    #ifndef _WIN32_
    if (  global_fd_led > 0 ) {
        statusLED.which = SHARP_LED_SALARM;
        statusLED.status =  LED_SALARM_OFF ;
        ioctl (global_fd_led, SHARP_LED_SETSTATUS, &statusLED);
    }
    #endif
}
*/

/*
void CallManager::startRing()
{
    kdDebug() << "CallManager::startRing()" << endl;
    // FIXME: implement buttons for av500
//    #ifndef _WIN32_
//    if ( global_fd_led > 0 ) {
//        statusLED.which = SHARP_LED_SALARM;
//        statusLED.status =  LED_SALARM_ON ;
//        ioctl (global_fd_led, SHARP_LED_SETSTATUS, &statusLED);
//    }
//    #endif
//    ringTimer->start( ringTime_1, TRUE );
}
*/
/*
void CallManager::ringTimeout( void )
{
    kdDebug() << "CallManager::ringTimeout()" << endl;
    
    if( isRingingTone )
    {
        if( ringCount < 5 )
        {
            
            QString wavFile = "KDE_Beep_Phone_2.ogg";
            
            useWav = true;
            
    std_dir = KGlobal::dirs();
            if ( !std_dir->exists( std_dir->findResourceDir("sound",wavFile) ) )
                useWav = false;
#ifdef DESKTOP_VERSION

            useWav = true;
#endif
            
            ringCount++;
            if ( useWav )
            {
                
                //KNotifyClient::Instance("kcall");
                //KNotifyClient::event("incoming-call");
                KAudioPlayer::play(wavFile);
                kdDebug() << "RING WAV " << endl;
            }
            else
            {
#ifndef DESKTOP_VERSION
                ////Sound::soundAlarm ();
#endif

                kdDebug() << "RING SOUND ALARM" << endl;
            }

//            ringTimer->start( ringTime_1, TRUE );
        }
        else
        {
            ringCount = 0;
            kdDebug() << "Ringing tone silent" << endl;
//            ringTimer->start( ringTime_2, TRUE );
        }
    }
}
*/
void CallManager::acceptCallTimeout( void )
{
    kdDebug() << "CallManager::acceptCallTimeout()" << endl;
    if( !audio->isAudioOn() )
    {
//        if( isRingingTone )
//        {}
        audio_fd = ::open( "/dev/dsp", O_WRONLY | O_NONBLOCK );
        if( audio_fd == -1 )
        {
            QMessageBox::information( 0, "Accept Call",
                                      "Close any program which might be using soundcard \nand then retry accepting call." );
            return;
        }
        else
        {
            ::close( audio_fd );
        }
    }

    if( audio->checkCodec( member ) )
    {
        audio->setCurrentCall( call );
        audio->attachToCallMember( member );
        member->acceptInvite(
            audio->audioOut().message( audio->getRtpCodec(),
                                       audio->getVideoRtpCodec(), audio->getBodyMask() ),
            MimeContentType( "application/sdp" ) );
    }
    else
    {
        member->notAcceptableHere();
        QMessageBox::information( 0, "Accept Call",
                                  "Accepted codec not found." );
    }
}

void CallManager::prepareForDelete( void )
{
    kdDebug() << "CallManager::prepareForDelete()" << endl;
    // inform GUI
    ////QCopEnvelope e("QPE/SipTel", "deleteCall(QString)");
    ////e << call->getCallId();

    emit readyForDelete(call->getCallId());
}

#include "callmanager.moc"
