#include <qstring.h>
#include <qdatetime.h>
#include <qdom.h>
#include <qfile.h>
#include <qstringlist.h>

#include <kdebug.h>
#include <kstandarddirs.h>

#include "diallist.h"
#include "sipcall.h"
#include "call.h"


DialList::DialList( unsigned int ml = 10 )
{
    kdDebug() << "DialList::DialList()" << endl;
    
    QValueList<Call> lastcalls;
    QValueList<Call> calls;
    maxlength = ml;
    dialListFilename = locateLocal("appdata", "calllist.lst");
    
    openList();
}

DialList::~DialList()
{
    kdDebug() << "DialList::~DialList()" << endl;
    
    //saveList();
}

QString DialList::getCallId(unsigned int callIndex)
{
    kdDebug() << "DialList::getCallId()" << endl;
    if(callIndex < lastcalls.size()){
        return lastcalls[callIndex].callId;
    }
    return false;
}

Call* DialList::getCall(unsigned int callIndex)
{
    kdDebug() << "DialList::getCall()" << endl;
    if(callIndex < lastcalls.size())
    {
        return &lastcalls[callIndex];
    }
    else
        return 0;
}

QStringList DialList::getLastCallsAsString()
{
    kdDebug() << "DialList::getLastCallsAsString()"<< endl;

    QStringList tmpList;
    
    unsigned int i;
    for( i=0 ; i < lastcalls.size() ; i++)
    {
        QString tmp;
        if(lastcalls[i].callerName == "")
        {    
            tmp  = lastcalls[i].callerNr;
        }else
        {
            tmp = lastcalls[i].callerName;
        }
        
        tmp += " (";
        if(QDate::currentDate() != lastcalls[i].date)
        {
            tmp += lastcalls[i].date.toString("dd.MM.yyyy");
            tmp += " ";
        }
        tmp += lastcalls[i].time.toString("hh:mm:ss");
        tmp += ")";
        
        tmpList.append(tmp);
    }
    
    return tmpList;
}

QValueList<Call> DialList::getLastCalls()
{
    kdDebug() << "DialList::getLastCallsAsString()"<< endl;

    return lastcalls;
}

QValueList<Call> DialList::getAllCalls(unsigned int length)
{
    kdDebug() << "DialList::getLastCallsAsString()"<< endl;

    QValueList<Call> tmp;
    if(length > 0)
    {
        if(calls.size() < length)
            length = calls.size();
        
        unsigned int i;
        for(i = 0; i < length; i++)
        {
            tmp.push_front(calls[i]);
        }
    }else
    {
        tmp = calls;
    }
    return tmp;
}

void DialList::insertCall(SipCall *ncall, int type=0, QString dialNr="")
{
    kdDebug() << "DialList::insertCall()" << endl;
    
    Call callTemp;
    
    //in case of incoming call
    if(type == 1)
    {
        SipUri uri = ncall->getContactUri();
        SipCallMember* scm = ncall->getMemberList();
        SipUri uri2 = scm->getUri();
        
        callTemp.callId             = ncall->getCallId();
        callTemp.callerNr           = uri2.uri();
        callTemp.callerName         = "";
        callTemp.subject            = ncall->getSubject();
        callTemp.type               = type;
        callTemp.date               = QDate::currentDate();
        callTemp.time               = QTime::currentTime();
        callTemp.duration           = 0;
    }else 
    
    //in case of outgoing call
    if(type == 2)
    {
        callTemp.callId             = ncall->getCallId();
        callTemp.callerNr           = dialNr;
        callTemp.callerName         = "";
        callTemp.subject            = ncall->getSubject();
        callTemp.type               = type;
        callTemp.date               = QDate::currentDate();
        callTemp.time               = QTime::currentTime();
        callTemp.duration           = 0;
    }    
    
    if(callTemp.callerNr == "")
        callTemp.callerNr = "unknown";
        
    bool exist = false;
    
    for(unsigned int i = 0; i < lastcalls.size(); i++)
    {
        if(lastcalls[i].callerNr == callTemp.callerNr)
        {
            exist = true;
            
            lastcalls[i].date = QDate::currentDate();
            lastcalls[i].time = QTime::currentTime();
            
        }
    }
    
    if(exist == false)
    {
        if(maxlength <= lastcalls.size())
        {    
            lastcalls.pop_front();
        }
        
        lastcalls.push_back(callTemp);
    }
    
    calls.push_front(callTemp);
    
    saveList();
}

void DialList::delList()
{
    kdDebug() << "DialList::delList()" << endl;
    
    for(unsigned int i = 0; i<lastcalls.size(); i++){
        lastcalls.pop_front();
    }
}

void DialList::openList()
{
    kdDebug() << "DialList::openList()" << endl;
    QString filename = dialListFilename;  
    calls.clear();
    lastcalls.clear();
    
    
    QDomDocument doc("KCallCalls");
    QFile f(filename);
    if (!f.open(IO_ReadOnly))
    {    
    }
    if (!doc.setContent(&f)) 
    {
        f.close();
    }
    f.close();
    
    QDomNodeList domCalls = doc.elementsByTagName("call");
    for(uint i=0;i<domCalls.count();++i) {
        QDomElement c = domCalls.item(i).toElement();
        if (!c.isNull()) {
            loadCall(&c);
        }
    }
}

void DialList::loadCall(QDomElement *c)
{
    kdDebug() << "DialList::loadCall()" << endl;
    
    QString callId = c->attribute("callid","");
    QString callerNr = c->attribute("callernr","unknown");
    QString callerName = c->attribute("callername","");
    QString subject = c->attribute("subject","");
    int type = (c->attribute("type","")).toInt();
    QDate date = QDate::fromString(c->attribute("date"));
    QTime time = QTime::fromString(c->attribute("time"));
    int duration = (c->attribute("duration","0")).toInt();
    
    reInsertCall(callId, callerNr, callerName, subject, type, date, time, duration);
    
}

void DialList::reInsertCall(QString callId, QString callerNr, QString callerName, QString subject, int type, QDate date, QTime time, int duration)
{
    kdDebug() << "DialList::reInsertCall()" << endl;
    
    Call callTemp;
    
    callTemp.callId = callId;
    callTemp.callerNr = callerNr;
    callTemp.callerName = callerName;
    callTemp.subject = subject;
    callTemp.type = type;
    callTemp.date = date;
    callTemp.time = time;
    callTemp.duration = duration;
        
    bool exist = false;
    
    for(int i = 0; i < lastcalls.size(); i++)
    {
        if(lastcalls[i].callerNr == callTemp.callerNr)
        {
            exist = true;
        }
    }
    
    if(exist == false)
    {
        if(maxlength <= lastcalls.size())
        {    
            lastcalls.pop_front();
        }
        
        lastcalls.push_back(callTemp);
    }
    
    calls.push_front(callTemp);
}

bool DialList::saveList()
{
    kdDebug() << "DialList::saveList()" << endl;
      
    QString filename = dialListFilename;
    QDomDocument doc("KCallCalls");
    QDomElement set = doc.createElement("calllist");
    doc.appendChild(set);

    for(uint i=0; i<calls.count();++i) 
    {
        saveCall(&calls[i],&doc,&set);
    }
    
    QFile xmlfile(filename);
    if (!xmlfile.open(IO_WriteOnly)) {
        kdDebug() << "Error while loading " << filename << endl;
        return false;
    }
    //kdDebug() << doc.toString();
    QTextStream ts(&xmlfile);
    doc.documentElement().save(ts,2);
    xmlfile.close();

    return true;
}

void DialList::saveCall(Call *tmpcall, QDomDocument *doc, QDomElement *parent)
{
    kdDebug() << "DialList::saveCall()" << endl;
    
    QDomElement callElement = doc->createElement("call");
    callElement.setAttribute("callid",tmpcall->callId);
    callElement.setAttribute("callernr",tmpcall->callerNr);
    callElement.setAttribute("callername",tmpcall->callerName);
    callElement.setAttribute("subject",tmpcall->subject);
    callElement.setAttribute("type",tmpcall->type);
    callElement.setAttribute("date",tmpcall->date.toString());
    callElement.setAttribute("time",tmpcall->time.toString());
    callElement.setAttribute("duration",tmpcall->duration);
    parent->appendChild(callElement);
}
