#include "kcall.h"

#include <klocale.h>
#include <kdebug.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <klineedit.h>
#include <kstdaction.h>
#include <knotifyclient.h>
#include <kapplication.h>
#include <kpopupmenu.h>
#include <kconfigdialog.h>

#include <qtimer.h>
#include <qpixmap.h>
#include <qcursor.h>

#include "kcallsipsettings.h"
#include "sipprefs.h"

#include "kcallcore.h"
#include "dialdialog.h"
#include "diallist.h"
#include "call.h"
#include "incomingdialog.h"


KCall::KCall()
     : KSystemTray()
     , m_isBlinking(false)
     , m_isBlinkIcon(false)
{
    m_blinkTimer = new QTimer(this, "m_blinkTimer");
    connect(m_blinkTimer, SIGNAL(timeout()), this, SLOT(slotBlink()));
    
    paintGui();
    mkcall = new KCallCore();
    
    kCallGuiRunning = mkcall->getGuiStatus();
    if(kCallGuiRunning)
        kdDebug() << "KCallGui is running!" << endl;
    else
        kdDebug() << "KCallGui is not running!" << endl;
    
    connect(mkcall, SIGNAL(sigGuiStatus(bool)), 
            this, SLOT(guiStatus(bool)));
        
    connect(mkcall,SIGNAL(GuiIncomingCall(QString, QString, QString)), 
            this, SLOT(slotStartIncomingCall(QString, QString, QString)));
            
    connect(mkcall,SIGNAL(registrationChanged(int)),
            this, SLOT(slotRegistrationChanged(int)));
        
    incomingdlg = new IncomingDialog(this);
    connect(incomingdlg, SIGNAL(accept(QString)), this, SLOT(acceptCall(QString)));
    connect(incomingdlg, SIGNAL(reject(QString)), this, SLOT(rejectCall(QString)));
    connect(incomingdlg, SIGNAL(hangup(QString)), this, SLOT(hangupCall(QString)));
    connect(incomingdlg, SIGNAL(dial()), this, SLOT(slotDialDlg()));

    
    //ringTimer = new QTimer;
    //connect(ringTimer, SIGNAL(timeout()), this, SLOT(ringAction()));
    
    //dcopClient = new DCOPClient();
    //dcopClient->send("kicker","Panel","restart()","");
    //dcopClient->send("kcall", "KCallCore", "registrate()", "");
    //mkcall->registrate();
    //dcopClient->send("kcall","KCallGuiCore","testing()","");
}


KCall::~KCall()
{
    kdDebug() << "KCall::~ KCall()" << endl;
    delete mkcall;
}


void KCall::mousePressEvent(QMouseEvent *event)
{
    kdDebug() << "KCall::mousePressEvent()" << endl;
    
    if (event->button() == LeftButton) 
    {
        this->showDialList( event );
    }
    else if (event->button() == RightButton)
    {
        this->showMainMenu( event );
    }
    else if (event->button() == MidButton)
    {
    
    }
}


void KCall::paintGui()
{
    kdDebug() << "KCall::paintGui()" << endl;
    
    onlineIcon  = "phone_connect";
    offlineIcon = "phone_disconnect";
    bellIcon    = "phone_bell";
    
    setBackgroundMode(X11ParentRelative);
    
    slotRegistrationChanged(KCallCore::NotConnected);
}


void KCall::showDialList(QMouseEvent *pos)
{
    kdDebug() << "KCall::showDialList()" << endl;
    
    KPopupMenu *dialListMenu = new KPopupMenu();
    dialListMenu->insertTitle(i18n("Call List"));
    QStringList lastCalls = mkcall->mDialList->getLastCallsAsString();
    
    for(QStringList::Iterator it = lastCalls.begin(); it != lastCalls.end(); ++it)
    {
        QString tmpCallTypeIcon = "forward";
        dialListMenu->insertItem(SmallIcon(tmpCallTypeIcon), *it, this, SLOT(slotDialDlg()));
    }
    
    /*for(int i = 0; i < mkcall->dialListLength; i++)
    {
        Call *tmpCall = mkcall->mDialList->getCall(i);
        if (tmpCall=0)
            DO SOMETHING USEFUL
        QString tmpCallerNr = tmpCall->callerNr;
        int tmpCallType = tmpCall->type;
        
        QString datetime = "";
        QString tmpCallTypeIcon;
        QDate tmpCallDate = tmpCall->date;
        QTime tmpCallTime = tmpCall->time;
        
        if(tmpCallDate != QDate::currentDate())
        {
            datetime += tmpCallDate.toString("dd.MM.yyyy");
            datetime += " ";
        }
        
        datetime += tmpCallTime.toString("hh:mm:ss");
        
        switch(tmpCallType)
        {
            case 1:
                //Incomming
                tmpCallTypeIcon = "forward";
                break;
            case 2:
                //Outgoing
                tmpCallTypeIcon = "back";
                break;
            default:
                //unkown
                tmpCallTypeIcon = "help";
                break;
        }
        
        if(tmpCall->type > 0)
        {
            QString tmpCallInfo = tmpCallerNr+" ("+datetime+")";
            kdDebug() << tmpCallInfo << endl;
            dialListMenu->insertItem(SmallIcon(tmpCallTypeIcon), tmpCallInfo, this, SLOT(slotDialDlg()));
        }else
        {
            i = mkcall->dialListLength;
        }
    }*/
    dialListMenu->popup( pos->globalPos() );
}


void KCall::showMainMenu(QMouseEvent *pos)
{
    kdDebug() << "KCall::showMainMenu()" << endl;
    
    KPopupMenu* mainMenu = new KPopupMenu(this);
    
    //contextMenuAboutToShow( mainMenu );            
    int buttonIdmain[10];
    buttonIdmain[0] = mainMenu->insertItem( SmallIcon( "kcall" ), i18n("&Open KCall"),
                                            this, SLOT( slotOpenKCall() ) );
    mainMenu->insertSeparator();
    buttonIdmain[1] = mainMenu->insertItem( SmallIcon( "configure" ), i18n( "&Configure KCall..." ),
                                            this, SLOT( slotPreferences() ) );
    mainMenu->insertSeparator();
    buttonIdmain[2] = mainMenu->insertItem( i18n( "Dial a Phonenumber" ),
                                            this, SLOT( slotDialDlg() ) );
    buttonIdmain[3] = mainMenu->insertItem( regButtonText ,
                                            this, SLOT( slotUnRegistrate() ) );
    buttonIdmain[4] = mainMenu->insertItem( SmallIcon("exit"),i18n( "Quit" ),
                                            kapp, SLOT( quit() ) );
    
    mainMenu->popup( pos->globalPos() );
}

void KCall::startBlink( const QPixmap& iconOn, const QPixmap& iconOff )
{
    m_blinkIconOn = iconOn;
    m_blinkIconOff = iconOff;
    if ( m_blinkTimer->isActive() == false )
    {
        m_isBlinkIcon = true;
        m_isBlinking = true;
        m_blinkTimer->start( 1000, false );
    }
    else
    {
        m_blinkTimer->stop();
        m_isBlinkIcon = true;
        m_isBlinking = true;
        m_blinkTimer->start( 1000, false );
    }
}

void KCall::stopBlink()
{
    if (m_blinkTimer->isActive() )
        m_blinkTimer->stop();

    m_isBlinkIcon = false;
    m_isBlinking = false;
}

void KCall::slotBlink()
{
    setPixmap( m_isBlinkIcon ? m_blinkIconOn : m_blinkIconOff );
    m_isBlinkIcon = !m_isBlinkIcon;
}

void KCall::slotOpenKCall()
{
    if (!kCallGuiRunning)
    {
        QString error;
        KApplication::startServiceByDesktopName(
                "kcall", 
                QString::null, &error );
        if (!error.isEmpty())
            KMessageBox::error(0, error);
     }
     else
     {
        /// TODO: bring kcall to front
     }
}


void KCall::slotAbout()
{
    KMessageBox::information(0, i18n("This is an about box"));
}


void KCall::slotHelp()
{
    KMessageBox::information(0, i18n("This is a help box"));
}


void KCall::slotPreferences()
{
    kdDebug() << "KCall::slotPreferences()" << endl;
    if (!kCallGuiRunning)
    {
       if(KConfigDialog::showDialog("settings"))
         return;
       KConfigDialog *dialog = new KConfigDialog(this, "settings",
            KCallSipSettings::self(), KDialogBase::IconList);
       dialog->addPage(new SIPPrefs(), KCallSipSettings::self(), i18n("SIP"), "kcall");
       //dialog->addPage(new General(0, "General"), i18n("General") );
       //dialog->addPage(new Appearance(0, "Style"), i18n("Appearance") );
       connect(dialog, SIGNAL(settingsChanged()), this, SLOT(slotSettingsChanged()));
       dialog->show();
    }
    else
    {
        kdDebug() << "dcop" << endl;
        mkcall->dcopClient->send("kcall", "KCallGuiCore", "showSettingsDialog()", "");
        //KMessageBox::information(0, i18n("This is not implemented yet."));
    }
}

void KCall::slotSettingsChanged()
{
    mkcall->settingsChanged();
}

void KCall::slotDialDlg()
{
    kdDebug() << "KCall::slotDialDlg()" << endl;
    
    DialDialog *dialdlg = new DialDialog;
    /*if(num != "")
    {    
        dialdlg->nrInput->setText(num);
    }
    */
        
    bool res;
    res = dialdlg->exec();
    
    // @FIXME Mike: what is this about? why is your name within the code?
    if(res){
        QString callId = mkcall->dial(dialdlg->nrInput->text());
        QString caller = "Mike Hauth";
        QString subject = "Hallo Welt!";
        slotStartIncomingCall(callId, caller, subject);
    }
}


void KCall::slotStartIncomingCall(QString callid, QString caller, QString subject)
{
    kdDebug() << "KCall::slotStartIncoming()" << endl;
    
    connect(mkcall,SIGNAL(callEnded(QString)), this, SLOT(slotStopIncomingCall(QString)));
    
    setPixmap( loadIcon( bellIcon ));
    
    if( kCallGuiRunning )
    {
        kdDebug() << "StartIncoming:KCallGui is running!" << endl;
        
        QByteArray  callData;
        
        QDataStream senddataA(callData, IO_WriteOnly);
        senddataA << callid << caller << subject << 1;         
        
        mkcall->dcopClient->send("kcall", "KCallGuiCore", "newCall(QString, QString, QString, int)", callData);

    }
    else
    {
        kdDebug() << "StartIncoming:KCallGui is not running!" << endl;
        
        incomingdlg->newCall(callid);
        incomingdlg->show();
    }
    //KNotifyClient::Instance();
    //KNotifyClient::event("incoming-call");
    //KNotifyClient::event(winId(), "incoming-call", "Anruf");
}

void KCall::slotStopIncomingCall(QString callid)
{
    kdDebug() << "KCall::slotStopIncomingCall()" << endl;
    if( kCallGuiRunning )
    {
        kdDebug() << "StopIncoming:KCallGui is running!" << endl;
        
        mkcall->dcopClient->send("kcall", "KCallGuiCore", "cancelCall(QString)", callid);
        setPixmap( loadIcon( onlineIcon) );
    }
    else
    {
        kdDebug() << "StopIncoming:KCallGui is not running!" << endl;
        if(incomingdlg){
            incomingdlg->cancelCall(callid);
            setPixmap( loadIcon( onlineIcon ) );
        }
    }
}


void KCall::slotRegistrationChanged(int newstatus)
{
    kdDebug() << "KCall:slotRegistrationChanged() " << newstatus << endl;
    switch (newstatus)
    {
        case KCallCore::Connecting:
            startBlink( loadIcon(onlineIcon), loadIcon(offlineIcon) );
            break;
        case KCallCore::Connected:
            stopBlink();
            setPixmap( loadIcon( onlineIcon ));
            regButtonText = i18n("Disconnect");
            break;
        case KCallCore::Disconnecting:
            startBlink( loadIcon(onlineIcon), loadIcon(offlineIcon) );
            break;
        case KCallCore::NotConnected:
        default:
            stopBlink();
            setPixmap( loadIcon( offlineIcon ));
            regButtonText = i18n("Connect to Server");
    }
}

void KCall::slotUnRegistrate()
{
    kdDebug() << "KCall::slotUnRegistrate()" << endl;
    if(mkcall->getRegistration())
    {
        mkcall->unregistrate();
        //dcopClient->send("kcall", "KCallCore", "unregistrate()", "");
    }else
    {
        mkcall->registrate();
        //dcopClient->send("kcall", "KCallCore", "registrate()", "");
    }
}

void KCall::acceptCall(QString callId)
{
    kdDebug() << "KCall::acceptCall()" << endl;
    
    mkcall->acceptCall( callId );
}

void KCall::rejectCall(QString callId)
{
    kdDebug() << "KCall::rejectCall()" << endl;
    
    mkcall->rejectCall( callId );
}

void KCall::hangupCall(QString callId)
{
    kdDebug() << "KCall::hangupCall()" << endl;
    
    mkcall->hangUp( callId );
}

void KCall::slotQuit()
{
    kdDebug() << "KCall::slotQuit()" << endl;
    
    exit(0);
}

void KCall::guiStatus(bool s)
{
    kdDebug() << "KCall::guiStatus()" << endl;
    
    kCallGuiRunning = s;
}

#include "kcall.moc"
