/***************************************************************************
 *   Copyright (C) 2004 by Eva Brucherseifer                               *
 *                         eva.brucherseifer@basyskom.de                   *
 *   Copyright (C) 2004 by Mike Hauth                                      *
 *                                                                         *
 ***************************************************************************/

#include <qtimer.h>
//#include <qbytearray.h>
//#include <qdatastream.h>

#include <kdebug.h>
#include <kconfig.h>
#include <kaudioplayer.h>
#include <kglobal.h>
#include <kapp.h>
#include <knotifyclient.h>
#include <kwin.h>

#include "callaudio.h"
#include "sipclient.h"
#include "sipregister.h"
#include "sipcall.h"
#include "kphoneconst.h"
#include "sdp.h"

#include "callmanager.h"
#include "ksipauthentication.h"
#include "kcallsipsettings.h"
//#include "kcallbackendsip.h"

#include "kcallcore.h"

KCallCore::KCallCore() : QObject(), DCOPObject("KCallCore"), audio(0), client(0), user(0), reg(0), stun_timer(0), reg_timer(0), b_registered(false)
{    
    KCallSipSettings::self()->readConfig();

    sipauthentication = new KSipAuthentication();
    
    //Init Generalsettings
    readGeneralSettings();

    //Init Audiosettings
    audio = new CallAudio();
	readAudioSettings();
   
    ringTimer = new QTimer;
    connect(ringTimer, SIGNAL(timeout()), this, SLOT(ringAction()));
    
    /*
    ////QCopChannel* telChannel = new QCopChannel( "QPE/SipTel", this );    
    ////connect(telChannel, SIGNAL(received(const QCString &, const QByteArray &)),
    ////        this, SLOT(message(const QCString &, const QByteArray &)) );

    ////QCopEnvelope e("QPE/SipTel", "registrationStatusChanged(int)");
    ////e << SipRegister::NotConnected;
    */
            
    //Registrate Client @ Server
    //this->registrate();
    
    int dialListLength = KCallSipSettings::maxLength();    
    mDialList = new DialList(dialListLength);
    
    ///////////////////////////////////////////////
    dcopClient = new DCOPClient();
    dcopClient->attach();
    
    registrateAtGui(true);
}

KCallCore::~KCallCore()
{
    registrateAtGui(false);

    unregistrate();
    delete audio;
    delete sipauthentication;
    delete mDialList;
    delete dcopClient;
}

void KCallCore::cleanupCallManagers()
{
    kdDebug() << "KCallCore::cleanupCallManagers()" << endl;
    
    QMap<QString,CallManager*>::Iterator it;
    
    for( it = callManagerMap.begin(); it != callManagerMap.end(); ++it )
        (*it)->hangupCall();
        
    for( it = callManagerMap.begin(); it != callManagerMap.end(); ++it )
        delete *it;
        
    callManagerMap.clear();
}

void KCallCore::timerTick()
{
    //kdDebug() << "KCallCore::timerTick()" << endl;

    client->doSelect( false );
}

void KCallCore::stun_timerTick()
{
    kdDebug() << "KCallCore::stun_timerTick()" << endl;

    client->sendStunRequest();
}

void KCallCore::checkIfRegistered()
{
    if (reg)
        if (reg->getRegisterState()!=SipRegister::NotConnected)
            return;
    registrate();
}



void KCallCore::registrate()
{
    kdDebug () << "KCallCore::registrate()" << endl;
    
    unregistrate();

    readAudioSettings();
    readGeneralSettings();

    // setup client
    QString socketStr = KCallSipSettings::socketMode();
    int socketPort = KCallSipSettings::socketPort();
    client = new SipClient( 0, 0, socketPort, looseRoute, strictRoute, socketStr );
    readClientSettings();

    // react on incoming calls
    connect(client, SIGNAL( incomingCall( SipCall*, QString)),
            this, SLOT( incomingCall( SipCall*, QString)));
    
    // setup user
    user = new SipUser( client, SipUri( userUri ) );
    kdDebug() << "SIP URI: " <<  userUri << endl;

    // STUN
    bool useStun = KCallSipSettings::useStun();
    QString stunSrv = "";
    if( useStun )
    {
        stunSrv = KCallSipSettings::stunServer();
        if( stunSrv.isEmpty() )
        {
            QString dname = user->getMyUri()->getHostname();
            stunSrv = dname;
            QString srv = client->getSRV( QString( "_stun._udp." ) + dname );
            if( !srv.isEmpty() )
                stunSrv = srv;
            stunSrv += ":3478";
        }
        else
            if( !stunSrv.contains( ':' ) )
                stunSrv += ":3478";
    }
    kdDebug() << "Using STUN Server: " << stunSrv << endl;

    // get registration settings
    QString userdefaultproxy;
    if (KCallSipSettings::useDifferentProxy())
        userdefaultproxy = KCallSipSettings::sipServer();
    if( userdefaultproxy.lower() == "sip:" )
        userdefaultproxy = "";
    kdDebug() << "SIP Proxy: " << userdefaultproxy << endl;
        
    double qvalue = KCallSipSettings::qValue();
    int expires = KCallSipSettings::registrationExpiresTime();
    if( expires == 0 )
        expires = -1;

    // setup registration object
    reg = new SipRegister( user, SipUri(userdefaultproxy), expires, QString::number(qvalue) );
    if (!reg)
    {
        kdDebug() << "Error: Couldn't create SipRegister" << endl;
        return;
    }
    connect(reg, SIGNAL(statusUpdated()), this, SLOT(updateRegistrationStatus()));

    // start stun
    if( useStun )
    {
        audio->setStunSrv( stunSrv );
        client->sendStunRequest( stunSrv );
        stun_timer = new QTimer( this );
        connect( stun_timer, SIGNAL( timeout() ), this, SLOT( stun_timerTick() ) );
        int timeout = KCallSipSettings::requestPeriod();
        if( timeout > 0 )
            stun_timer->start( timeout * 1000 );
    }
 
    // update client and user and progress registration
    client->updateIdentity( user, reg->getOutboundProxy() );
    
    user->getMyUri()->setProxyUsername( KCallSipSettings::userName() );
    if( KCallSipSettings::autoRegister()=="Yes" )
    {
        if( useStun )
            reg->setAutoRegister( true );
        else
            reg->requestRegister();
    }
    else
        reg->setAutoRegister( false );
    if( KCallSipSettings::autoRegister()=="Yes" && useStun)
        reg->setAutoRegister(true);
    if (!username.isEmpty() && !password.isEmpty())
    	reg->requestRegister(username,password);
    else
	reg->requestRegister();
    b_registered = true;    
    
    reg_timer = new QTimer( 0 );
    connect( reg_timer, SIGNAL( timeout() ), this, SLOT( timerTick() ) );
    reg_timer->start( 100 );
}

void KCallCore::unregistrate()
{
    kdDebug() << "KCallCore::unregistrate()" << endl;
    
    cleanupCallManagers();
    
    delete reg_timer;
    reg_timer=0;
    delete stun_timer;
    stun_timer=0;

    if( reg )
        if( reg->getRegisterState() == SipRegister::Connected )
            reg->requestClearRegistration();
    
    disconnect(client, SIGNAL( incomingCall( SipCall*, QString)), 
                 this, SLOT( incomingCall( SipCall*, QString)));
    disconnect(reg, SIGNAL(statusUpdated()), 
                 this, SLOT(updateRegistrationStatus()));
    
    delete client;
    client=0;
    delete user;
    user=0;
    delete reg;
    reg=0;
 
    emit registrationChanged(NotConnected);
    
    ////QCopEnvelope e("QPE/SipTel", "registrationStatusChanged(int)");
    ////e << SipRegister::NotConnected;
}

void KCallCore::updateRegistrationStatus()
{
    kdDebug() << "KCallCore::updateRegistrationStatus()" << endl;
    if (reg)
    {
        int regstate = reg->getRegisterState();
        switch (regstate)
        {
            case SipRegister::Connected:
                emit registrationChanged(Connected);
                break;
            case SipRegister::NotConnected:
                emit registrationChanged(NotConnected);
                break;
            case SipRegister::Disconnecting:
                emit registrationChanged(Disconnecting);
                break;
            default:
                emit registrationChanged(Connecting);
        }
        ////QCopEnvelope e("QPE/SipTel", "registrationStatusChanged(int)");
        ////e << reg->getRegisterState();

        if( regstate != SipRegister::AuthenticationRequired &&
                regstate != SipRegister::AuthenticationRequiredWithNewPassword )
        {
            kdDebug() << "reg == true2" << endl;
            return;
        }
        QString authtype;
        switch( reg->getAuthenticationType() )
        {
        case SipRegister::DigestAuthenticationRequired:
            authtype = tr("Digest Authentication Request"); break;
        case SipRegister::BasicAuthenticationRequired:
            authtype = tr("Basic Authentication Request"); break;
        case SipRegister::ProxyDigestAuthenticationRequired:
            authtype = tr("Proxy Digest Authentication Request"); break;
        case SipRegister::ProxyBasicAuthenticationRequired:
            authtype = tr("Proxy Basic Authentication Request"); break;
        }
        QString server = reg->getServerUri().proxyUri();
        QString sipuri = user->getUri().uri();

        QString username = reg->getRegisterCall()->getProxyUsername();
        QString password = reg->getRegisterCall()->getPassword();
        //authreq.setUsername( username );
       if( password.isEmpty() || reg->getRegisterState() == SipRegister::AuthenticationRequiredWithNewPassword )
        {
            QStringList authdata = sipauthentication->newAuthData( server, sipuri, "Wrong authentication data" );
            if( authdata.count()==2 )
            {
                username = authdata[0];
                password = authdata[1];
                if( username.isEmpty() )
                    return;
                reg->getRegisterCall()->setProxyUsername( username );
                reg->getRegisterCall()->setPassword( password );
                reg->requestRegister( username, password );
            }
        }
        else
        {
            reg->requestRegister( username, password );
        }
    }else
    {
        kdDebug() << "reg == false2" << endl;
        ////QCopEnvelope e("QPE/SipTel", "registrationStatusChanged(int)");
        ////e << SipRegister::NotConnected;
    	emit registrationChanged(NotConnected);
    }
}

void KCallCore::incomingCall( SipCall *call, QString body )
{
    kdDebug() << "KCallCore::incomingCall()" << endl;
    
    // notify GUI
    /*QStringList members;
    SipCallMemberIterator it = call->getMemberList();
    do
        members.append( (*it)->getUri().uri() );
    while (++it);
    // incomingCall parameters: callid, subject, member uris
    kdDebug() << "" << endl;
    ////QCopEnvelope e("QPE/SipTel", "incomingCall(QString, QString, QStringList)");
    ////e << call->getCallId();
    ////e << call->getSubject();
    ////e << members;
    */
    
    if( call->getSubject() == QString::null )
    {
        call->setSdpMessageMask( body );
    }
    ////qDebug("incomingCall: %s", call->getSubject().latin1() );
    kdDebug() << "incomingCall..." << call->getCallId() << endl;
    
    resetAudio();

    CallManager *manager = new CallManager( 0, audio, call, true, 0 );
    callManagerMap.insert(call->getCallId(), manager);
    connect(manager, SIGNAL(readyForDelete(const QString& )), this, SLOT(deleteManager(const QString& )));
    
    mDialList->insertCall(call,1,"");
    
    emit GuiIncomingCall(call->getCallId(), call->getProxyUsername(), call->getSubject());
    startRing();
}

QString KCallCore::dial(const QString& num)
{
    kdDebug() << "KCallCore::dial()" << endl;
    
    if (!b_registered)
    {
        registrate();
        if (!b_registered)
            return false;
    }

    if (num.isEmpty())
    {
        kdDebug() << "KCallCore::dial: empty number" << endl;
        return false;
    }
    
    if (!client)
    {
        kdDebug() << "KCallCore::dial: client not registered at server" << endl;
        return false;
    }
    
    
    SipCall *newcall = new SipCall( user, QString::null, SipCall::StandardCall );
    newcall->setSubject( user->getUri().uri() );
    mDialList->insertCall(newcall, 2, num);
    
    resetAudio();
    
    CallManager *manager = new CallManager( sipauthentication, audio, newcall, false, 0 );
    manager->setRemote( num );
    callManagerMap.insert(newcall->getCallId(), manager);
    
    connect(manager, SIGNAL(readyForDelete(const QString& )), this, SLOT(deleteManager(const QString& )));
    manager->dial();
    return newcall->getCallId();
    //connect( widget, SIGNAL( redirectCall( const SipUri &, const QString & ) ),
    //		this, SLOT( redirectCall( const SipUri &, const QString & ) ) );
}

void KCallCore::deleteManager(const QString& callid)
{
    kdDebug() << "KCallCore::deleteManager()" << endl;

    emit callEnded(callid);
    stopRing();
    readyForDeleteManagers.push(callManagerMap[callid]);
    callManagerMap.remove(callid);
    QTimer::singleShot(0, this, SLOT(deleteManagerDelayed()));

}

void KCallCore::deleteManagerDelayed()
{
    //    qDebug("QSipTelQCop::deleteManager %s", callid.latin1());
    CallManager* cm;
    while ( (cm=readyForDeleteManagers.pop())!=0)
        delete cm;
}

void KCallCore::readGeneralSettings()
{
    kdDebug() << "KCallCore::readGeneralSettings()" << endl;
    
    
    if(Sip::getLocalAddress().isEmpty()){
        kdDebug() << "Local Address is empty" << endl;
    }
    
    if (!KCallSipSettings::iPaddress().isEmpty())
	Sip::setLocalAddress( KCallSipSettings::iPaddress() );
     
    kdDebug() << "Sip::setLocalAddress() done" << endl;
    
    userUri = "\" "  
           + KCallSipSettings::fullName() 
           + "\" <sip:" 
           + KCallSipSettings::sipUriUserName()
           + "@" 
           + KCallSipSettings::sipUriDomain()
           + ">";
    if (userUri.isEmpty())
    {
        kdDebug() << "SIP URI is empty!" << endl;
        // FIXME
    }
    username = KCallSipSettings::userName();
    password = KCallSipSettings::password();
    sipauthentication->setAuthData(username, password);
}

void KCallCore::readClientSettings()
{
    kdDebug() << "KCallCore::readClientSettings()" << endl;
     
    if (!client)
        return;

    if( KCallSipSettings::useDifferentProxy() )
        client->setUseProxyDial( true );
    else
        client->setUseProxyDial( false );

    client->setCallForward( KCallSipSettings::callforward() );
    client->setCallForwardUri( SipUri( KCallSipSettings::forwardaddr() ) );
    client->setCallForwardMessage( KCallSipSettings::forwardmsg() );
    client->setMaxForwards( KCallSipSettings::maxforwards() );
    client->setBusyMessage( KCallSipSettings::busymsg() );

    if( KCallSipSettings::signalling() )
        client->setSymmetricMode( true );
    else
        client->setSymmetricMode( false );

    QString hvstr = KCallSipSettings::hideviamode();
    if( hvstr == "NoHide" )
        client->setHideViaMode( SipClient::DontHideVia );
    else if( hvstr == "HideHop" )
        client->setHideViaMode( SipClient::HideHop );
    else if( hvstr == "HideRoute" )
        client->setHideViaMode( SipClient::HideRoute );
}

void KCallCore::readAudioSettings()
{
    kdDebug() << "KCallCore::readAudioSettings()" << endl;
    
    if (!audio)
        return;
    
    //audio->readAudioSettings();
    audio->setOSSMode(KCallSipSettings::mode());
	audio->setOSSFilenameFS(KCallSipSettings::fragSizeWrite());
	audio->setOSSFilename2FS(KCallSipSettings::fragSizeRead());

	audio->readVideoSettings();
    
    QString codec = KCallSipSettings::codec();
    
    resetAudio();
        
    audio->setPayload( KCallSipSettings::sizeOfPayload() );
    
    if ( KCallSipSettings::media() == "Yes")
        audio->setSymMediaMode(true);
    else
        audio->setSymMediaMode(false);
}

void KCallCore::resetAudio()
{
    if (codec == "ULAW")
        audio->setCodec( codecULAW );
    else if (codec == "GSM")
        audio->setCodec( codecGSM );
    else if (codec == "ILBC")
        audio->setCodec( codecILBC );
        
}

SipRegister* KCallCore::getSipRegister()
{
    kdDebug() << "KCallCore::getSipRegister()" << endl;    

    return reg;
}


//Slots


void KCallCore::settingsChanged()
{
    kdDebug() << "void KCallCore::settingsChanged()" << endl;
    KCallSipSettings::self()->readConfig(); 
    registrate();
}

void KCallCore::identityChanged()
{
    kdDebug() << "void KCallCore::identityChanged()" << endl;
    KCallSipSettings::self()->readConfig(); 
    registrate();
}

void KCallCore::hangUp(QString callid)
{
    kdDebug() << "void KCallCore::hangUp()" << endl;
    
    if (callManagerMap.contains(callid))
        callManagerMap[callid]->hangupCall();
}

void KCallCore::acceptCall(QString callid)
{
    kdDebug() << "void KCallCore::acceptCall()" << endl;

    stopRing();
    if (callManagerMap.contains(callid))
        callManagerMap[callid]->acceptCall();

}

void KCallCore::rejectCall(QString callid)
{
    kdDebug() << "void KCallCore::rejectCall()" << endl;

    stopRing();
    if (callManagerMap.contains(callid))
        callManagerMap[callid]->hangupCall();
}

bool KCallCore::getRegistration()
{
    kdDebug() << "void KCallCore::getRegistration()" << endl;
    
    ////QCopEnvelope e("QPE/SipTel", "registrationStatusChanged(int)");
    if (!reg){
        kdDebug() << "reg==false" << endl;
        ////e << SipRegister::NotConnected;
        return false;
    }
    else
    {
        kdDebug() << "reg==true" << endl;
        ////e << reg->getRegisterState();
        return true;
    }
}

void KCallCore::quit()
{
    kdDebug() << "void KCallCore::quit()" << endl;
}

void KCallCore::startRing()
{
    kdDebug() << "KCallCore::startRing()" << endl;
    ringTimer->start(ringTime_1, true);
}

void KCallCore::stopRing()
{
    kdDebug() << "KCallCore::stopRing()" << endl;
    ringTimer->stop();
    system("artsshell terminate");
}

void KCallCore::ringAction()
{
    kdDebug() << "KCallCore::ringAction()" << endl;
    
    //temp. until knotify
    KStandardDirs *stdDir = new KStandardDirs();
    QString wavFile = "KDE_Beep_Phone_2.ogg";
    
    if ( stdDir->exists( stdDir->findResourceDir("sound",wavFile) ) )
    {    
        kdDebug() << "KCallCore::ringAction:ring!" << endl;
        
     //   KNotifyClient::event(winId(),"incoming-call");
        KNotifyClient::userEvent("Gestartet",1,1,"/opt/kde3/share/sounds/KDE_Beep_Phone_2.ogg","a");
    }
    
    ringTimer->start(ringTime_2, true);
}

bool KCallCore::getGuiStatus()
{
    kdDebug() << "KCallCore::checkKCallGuiStatus()" << endl;
    
    bool s = dcopClient->isApplicationRegistered("kcall");
    emit( sigGuiStatus(s) );
    return s;
}

bool KCallCore::registrateAtGui(bool state)
{
    kdDebug() << "KCallCore::registrateAtGui()" << endl;
    
    QByteArray  data;
    QDataStream senddata(data, IO_WriteOnly);
    senddata << state;         
    
    if(dcopClient->send("kcall", "KCallGuiCore", "kCallStatusReport(bool)", data))
        return true;
    else
        return false;
}

void KCallCore::guiStatusReport(bool status)
{
    kdDebug() << "KCallCore::guiStatusReport()" << endl;
    
    emit sigGuiStatus( status );
}





//TODO wtf is that good for?
//////just for xp
void KCallCore::restartKicker()
{
    KAudioPlayer::play("KDE_Beep_Phone_2.ogg");
    dcopClient->send("kicker", "Panel", "restart()", "");
    emit Something();
}

void KCallCore::writeComment(){
    kdDebug() << "Hallo Welt" << endl;
}

#include "kcallcore.moc"
