/***************************************************************************
 *   Copyright (C) 2004 by Eva Brucherseifer                               *
 *                         eva.brucherseifer@basyskom.de                   *
 *   Copyright (C) 2004 by Mike Hauth                                      *
 *                                                                         *
 ***************************************************************************/

#ifndef KCALLCORE_H
#define KCALLCORE_H

#include <dcopclient.h>
#include <dcopobject.h>

#include <qobject.h>
#include <qstring.h>
#include <qmap.h>
#include <qstack.h>
#include <kconfig.h>
#include "diallist.h"

class CallAudio;
class SipClient;
class SipUser;
class SipCall;
class SipRegister;
class QTimer;
class KSipAuthentication;

class CallManager;

class KCallCore :  public QObject, public DCOPObject
{
    K_DCOP
    Q_OBJECT

    public:
        KCallCore();
        virtual ~KCallCore();

        void readGeneralSettings();
        void readAudioSettings();
        void readClientSettings();
        void resetAudio();
        bool getGuiStatus();
        SipRegister* getSipRegister();

        DialList* mDialList;

        void restartKicker();
        DCOPClient *dcopClient;

        enum RegisterState {
            NotConnected,
            Connecting,
            Connected,
            Disconnecting
            };

    protected:
        void cleanupCallManagers();
        bool registrateAtGui(bool state);

        CallAudio *audio;
        SipClient *client;
        SipUser *user;
        SipRegister *reg;
        QTimer *quitTimer;
        QTimer *stun_timer;
        QTimer *reg_timer;

        bool b_registered;

        /** maps call ids to CallManagers */
        QMap<QString, CallManager*> callManagerMap;
        QStack<CallManager> readyForDeleteManagers;

        KSipAuthentication* sipauthentication;

        QString codec;
        QString userUri;
        bool useStun;

        QTimer *ringTimer;

        QString username;
        QString password;

    k_dcop:

        void settingsChanged();
        void identityChanged();

        void registrate();
        void unregistrate();

        QString dial(const QString& num);
        void hangUp(QString callid);
        void acceptCall(QString callid);
        void rejectCall(QString callid);

        bool getRegistration();
        void quit();
        void guiStatusReport(bool status);

        void writeComment();//just for experiment

    protected slots:

        void timerTick();
        void stun_timerTick();

        void updateRegistrationStatus();
        void incomingCall( SipCall *call, QString body );

        void deleteManager(const QString& callid);
        void checkIfRegistered();

    protected slots:

        void deleteManagerDelayed();
        void startRing();
        void stopRing();
        void ringAction();

    signals:
        void GuiIncomingCall( QString, QString, QString );
        void callEnded( QString );
        void registrationChanged( int );
        void sigGuiStatus( bool );

        void Something();   //just for experiment

};
#endif
