/* 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/
//This file was modified by Eva Brucherseifer

#include <stdio.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qlabel.h>
#include <qframe.h>
#include <qlineedit.h>
#include <qhbuttongroup.h>
#include <qradiobutton.h>
#include <qpushbutton.h>
#include <qmessagebox.h>
#include <qlayout.h>

#include "../dissipate2/sipcall.h"
#include "../dissipate2/sipuser.h"
#include "../dissipate2/sipregister.h"
#include "../dissipate2/sipclient.h"
#include "kphoneconst.h"
#include "ksipauthentication.h"

KSipAuthenticationRequest::KSipAuthenticationRequest(
    const QString &server, const QString &sipuri, const QString &caption, QWidget *parent, const char *name )
        : QDialog( parent, caption, true )
{
    if (caption.isEmpty())
        setCaption("Enter Password");
    else 
        setCaption(caption);
    QVBox *vbox = new QVBox( this );
    vbox->setMargin( 3 );
    vbox->setSpacing( 3 );
    QVBoxLayout *vboxl = new QVBoxLayout( this, 5 );
    vboxl->addWidget( vbox );
    (void) new QLabel( "SipUri = " + sipuri, vbox );
    (void) new QLabel( "Proxy Server = " + server, vbox );
    (void) new QLabel( "Username:", vbox );
    usernameLineEdit = new QLineEdit( vbox );
    (void) new QLabel( "Password:", vbox );
    passwordLineEdit = new QLineEdit( vbox );
    passwordLineEdit->setEchoMode( QLineEdit::Password );
    passwordLineEdit->setFocus();
    QHBoxLayout *buttonBox;
    buttonBox = new QHBoxLayout( vboxl, 6 );
    okPushButton = new QPushButton( this, "ok button" );
    okPushButton->setText( "OK" );
    okPushButton->setDefault( TRUE );
    buttonBox->addWidget( okPushButton );
    cancelPushButton = new QPushButton( this, "cancel button" );
    cancelPushButton->setText( "Cancel" );
    cancelPushButton->setAccel( Key_Escape );
    buttonBox->addWidget( cancelPushButton );
    connect( okPushButton, SIGNAL( clicked() ), this, SLOT( accept() ) );
    connect( cancelPushButton, SIGNAL( clicked() ), this, SLOT( reject() ) );
}

KSipAuthenticationRequest::~KSipAuthenticationRequest( void )
{}

void KSipAuthenticationRequest::setUsername( const QString &newUsername )
{
    usernameLineEdit->setText( newUsername );
}

void KSipAuthenticationRequest::setPassword( const QString &newPassword )
{
    passwordLineEdit->setText( newPassword );
}

QString KSipAuthenticationRequest::username( void )
{
    return usernameLineEdit->text();
}

QString KSipAuthenticationRequest::password( void )
{
    return passwordLineEdit->text();
}


// =================================================================

KSipAuthentication::KSipAuthentication()
{
    execAuthreq = false;
    authFromFile=false;
}

KSipAuthentication::~KSipAuthentication( void )
{}

void KSipAuthentication::setAuthData(const QString& user, const QString& pw)
{
    username=user;
    password=pw;
    authFromFile=true;
}

QStringList KSipAuthentication::authData(const QString &server, const QString &sipuri, const QString &caption)
{
    QStringList auth;
    if (!authFromFile)
    {
        return newAuthData(server, sipuri, caption);
    }
    if( username.isEmpty() || password.isEmpty() )
        return auth;
    auth << username << password;
    return auth;
}

QStringList KSipAuthentication::newAuthData(const QString &server, const QString &sipuri, const QString &caption)
{
    QStringList auth;
        KSipAuthenticationRequest* dialog = new KSipAuthenticationRequest( server, sipuri, caption );
        dialog->setUsername( username );
        dialog->setPassword( password );
        if( !dialog->exec() )
        {
            delete dialog;
            return auth;
        }
        username = dialog->username();
        password = dialog->password();
        delete dialog;
    if( username.isEmpty() || password.isEmpty() )
        return auth;
    auth << username << password;
    return auth;
}


void KSipAuthentication::authRequest( SipCallMember *member )
{
    qDebug("KSipAuthentication::authRequest");
    if( member->getAuthState() != SipCallMember::authState_AuthenticationRequired &&
            member->getAuthState() != SipCallMember::authState_AuthenticationRequiredWithNewPassword )
        return;

    QString u = member->getCall()->getProxyUsername();
    QString p = member->getCall()->getPassword();
    if( execAuthreq )
        return;
    if( p.isEmpty() ||
            member->getAuthState() == SipCallMember::authState_AuthenticationRequiredWithNewPassword )
    {
        QString proxy = member->getCall()->getSipProxy();
        SipUri localuri = member->getCall()->localAddress();

        if (!authFromFile)
        {
            KSipAuthenticationRequest* dialog = new KSipAuthenticationRequest( proxy, localuri.uri(), QString::null );
            dialog->setUsername( u );
            dialog->setPassword( p );
            execAuthreq = true;
            if( !dialog->exec() )
            {
                delete dialog;
                execAuthreq = false;
                return;
            }
            u = dialog->username();
            p = dialog->password();
            username=u;
            password=p;
            delete dialog;
            execAuthreq = false;
        }
        if( username.isEmpty() || password.isEmpty() )
            return;
        member->getCall()->setPassword( password );
    }

    switch( member->getCallMemberType() )
    {
    case SipCallMember::Subscribe:
        member->sendRequestSubscribe( username, password );
        break;
    case SipCallMember::Notify:
        member->sendRequestNotify( username, password );
        break;
    case SipCallMember::Message:
        member->sendRequestMessage( username, password );
        break;
    case SipCallMember::Invite:
        member->sendRequestInvite( username, password );
        break;
    default:
        break;
    }
}

#include "ksipauthentication.moc"
